/*
    ldapdiff
    Copyright (C) 2000-2008 Thomas.Reith@rhoen.de

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <lber.h>
#include <ldap.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "config.h"

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "ldapdiff.h"

/* global variables */
char *profile  = NULL;
char *logfile  = NULL;
int   facility = -1;
int   loglevel = 0;

static void printheader()
{
 printf("%s Copyright (C) 2000-2008 %s\n",PACKAGE_STRING,PACKAGE_BUGREPORT);
 printf("This program comes with ABSOLUTELY NO WARRANTY\n");
 printf("This is free software, and you are welcome to redistribute it\n");
 printf("under certain conditions. For more information on this and how\n");
 printf("to apply and follow the GPL, see http://www.gnu.org/licenses/gpl.html\n");
}

static void printusage()
{
 printf("usage: ldapdiff [options] -p profile\n");
 printf("                 options: -c configfile\n");
 printf("                 options: -d loglevel 0|1|2\n");
 printf("                 options: -f ldiffile\n");
 printf("                 options: -l logfile\n");
 printf("                 options: -s local[0-7] (syslog facility)\n");
 printf("                 options: -v (print version and exit)\n");
}

int main(int argc,char* argv[])
{
 extern char     *optarg;
 extern int       optind;
 int              option;
 int              ldapdiffrc  = EXITLDOK;
 int              errflag     = 0;
 char            *cfile       = NULL;
 char            *ifile       = NULL;
 LDAP            *ld;
 struct s_ldif   *sldif       = NULL;
 struct s_ldif   *sldifstart  = NULL;
 struct s_mod    *smod        = NULL;
 struct s_schema *sschema     = NULL;
 time_t           tstart;
 time_t           tend;

 time(&tstart);

 while ((option = getopt(argc, argv, "c:d:f:l:p:s:vh")) != EOF){
  switch(option){
   case 'c':
        cfile    = optarg;
        break;
   case 'd':
        loglevel = atoi(optarg);
        break;
   case 'f':
        ifile    = optarg;
        break;
   case 'h':
        printheader();
        printusage();
        exit(EXITLDOK);
        break;
   case 'l':
        logfile  = optarg;
        break;
   case 'p':
        profile  = optarg;
        break;
   case 's':
        if((facility = ldifgetfacility(optarg)) == -1){
         errflag++;
        }
        break;
   case 'v':
        printheader();
        exit(EXITLDOK);
        break;
   default:
        errflag++;
  }
 }

 if(errflag != 0 || profile == NULL){
  printheader();
  printusage();
  exit(EXITLDERROR);
 }

 /* log version information, always */
 ldiflog(LOG1,"%s %s\n",PACKAGE_STRING,PACKAGE_BUGREPORT);
 ldiflog(LOG1,"profile:           [%s]",profile);

 if(cfile == NULL){
  cfile = LDALLOC(strlen(DATADIR) + strlen("/ldapdiff/") 
          + strlen(LDAPDIFFCONF) + 1,sizeof(char));
  sprintf(cfile,"%s/ldapdiff/%s",DATADIR,LDAPDIFFCONF);
 }

 ldifinitconf(cfile); 

 /* open ldap connection 1st time */
 ldifopen(&ld);
 /* close ldap connection 1st time */
 ldifclose(ld);

 /* read ldif input */
 ldifread(&sldif,ifile);
 if(sldif == NULL){
  ldiflog(LOG1,"no ldif entries found, normal exit...");
  exit(EXITLDOK);
 }
 sldifstart = sldif; 

 /* open ldap connection 2nd time */
 ldifopen(&ld);

 /* build the schema cache */
 ldifschemacreate(ld,&sschema);

 /* replace all aliases with the real attribute names */
 ldifschemareplacealiases(sldif,sschema);

 /* check filter for aliased name and equality method in attribute */
 ldiffiltercheck(sschema);

 /* compare ldif file against ldap server */
 ldifiterldif(ld,sldifstart,&smod,sschema);

 /* compare ldap server against ldif file */
 ldifiterldap(ld,sldifstart,&smod,sschema);

 ldiffree(&sldifstart);

 ldifstats(smod);

 if(strcmp(ldifgetgconf(CONFONLINEUPDATE),"yes") == 0){
  if(ldifmodify(ld,smod,sschema) != 0){
   ldapdiffrc = EXITLDWARNING;
  }
 }

 if(strcmp(ldifgetgconf(CONFOFFLINEUPDATE),"yes") == 0){
  ldifwrite(smod,sschema);
 }

 /* free the schema cache */
 ldifschemafree(&sschema);

 /* close ldap connection 1st time */
 ldifclose(ld);

 /* calculate execution time */
 time(&tend);
 ldiflog(LOG1,"time:   %10d seconds",tend - tstart);

 exit(ldapdiffrc);
}
